@@ -1,16 +1,18 @@
-from pydantic import BaseModel
-from fastapi.responses import JSONResponse
-from typing import Optional, List, Dict, Any
 # from fastapi import HTTPException
 # from fastapi.responses import JSONResponse, PlainTextResponse
 import datetime
 import json
-import requests
-from urllib.parse import urlparse, quote
-from rdflib import ConjunctiveGraph, URIRef, Literal, BNode, RDF
+from typing import Any, Dict, List, Optional
+from urllib.parse import quote, urlparse
+
 import extruct
+import requests
 from fair_test.config import settings
+from fastapi.responses import JSONResponse
+from pydantic import BaseModel
 from pyld import jsonld
+from rdflib import RDF, BNode, ConjunctiveGraph, Literal, URIRef
+
 # pyld is required to parse jsonld with rdflib
 
 
@@ -123,7 +125,6 @@ class FairTestEvaluation(BaseModel):
             except Exception as e:
                 self.warn(f'Failed to reach the Harvester at {harvester_url}, using the built-in python harvester')
 
-        self.info(f'Checking if Signposting links can be found in the resource URI headers at {url}')
         # https://github.com/FAIRMetrics/Metrics/blob/master/MetricsEvaluatorCode/Ruby/metrictests/fair_metrics_utilities.rb#L355
         found_signposting = False
         html_text = None
@@ -138,7 +139,6 @@ class FairTestEvaluation(BaseModel):
             html_text = r.text
             if r.history:
                 # Extract alternative URIs if request redirected
-                self.info(f"Request was redirected to {r.url}.")
                 redirect_url = r.url
                 if redirect_url.startswith('https://linkinghub.elsevier.com/retrieve/pii/'):
                     # Special case to handle Elsevier bad redirections to ScienceDirect
@@ -146,45 +146,52 @@ class FairTestEvaluation(BaseModel):
 
                 self.data['redirect_url'] = redirect_url
                 if url == self.subject and not redirect_url in self.data['alternative_uris']:
-                    self.info(f"Adding {redirect_url} to the list of alternative URIs for the subject")
+                    self.info(f"Request was redirected to {redirect_url}, adding to the list of alternative URIs for the subject")
                     self.data['alternative_uris'].append(redirect_url)
                     if r.url.startswith('http://'):
                         self.data['alternative_uris'].append(redirect_url.replace('http://', 'https://'))
                     elif r.url.startswith('https://'):
                         self.data['alternative_uris'].append(redirect_url.replace('https://', 'http://'))
 
-            if 'link' in r.headers.keys():
-                signposting_links = r.headers['link']
-                found_signposting = True
-            if 'Link' in r.headers.keys():
-                signposting_links = r.headers['Link']
-                found_signposting = True
-            if found_signposting:
-                self.info(f'Found Signposting links: {str(signposting_links)}')
-                self.data['signposting'] = str(signposting_links)
-                # TODO: parse signposting links, get alternate and meta? https://signposting.org/FAIR
-                # return self.retrieve_metadata(str(signposting_links))
-            else:
-                self.info('Could not find Signposting links')
-        except Exception:
-            self.warn(f'Could not resolve the URL: {url}')
-            # Error: e.args[0]
+            # Handle signposting links headers https://signposting.org/FAIR
+            if r.links:
+                # Follow signposting links, this could create a lot of recursions (to be checked)
+                self.info(f'Found Signposting links: {str(r.links)}')
+                self.data['signposting_links'] = r.links
+                check_rels = ['alternate', 'describedby', 'meta']
+                # Alternate is used by schema.org
+                for rel in check_rels:
+                    if rel in r.links.keys():
+                        rel_url = r.links[rel]['url']
+                        if not rel_url.startswith('http://') and not rel_url.startswith('https://'):
+                            # In some case the rel URL provided is relative to the requested URL
+                            if r.url.endswith('/') and rel_url.startswith('/'):
+                                rel_url = rel_url[1:]
+                            rel_url = r.url + rel_url
+                        metadata_obj = self.retrieve_metadata(rel_url)
+                        if len(metadata_obj) > 0:
+                            return metadata_obj
+
+        except Exception as e:
+            self.warn(f"Error resolving the URL {url} : {str(e.args[0])}")
 
         self.info('Checking for metadata embedded in the HTML page returned by the resource URI ' + url + ' using extruct')
+        # TODO: support client-side JS generated HTML using Selenium https://github.com/vemonet/extruct-selenium
         try:
             extructed = extruct.extract(html_text.encode('utf8'))
             if url == self.subject:
                 self.data['extruct'] = extructed
-            self.info(f"Found metadata with extruct in the formats: {', '.join(extructed.keys())}")
             if len(extructed['json-ld']) > 0:
                 g = self.parse_rdf(extructed['json-ld'], 'json-ld', log_msg='HTML embedded JSON-LD RDF')
                 if len(g) > 0:
+                    self.info(f"Found JSON-LD RDF metadata embedded in the HTML with extruct")
                     return g
                 else:
                     metadata_obj = extructed['json-ld']
             if len(extructed['rdfa']) > 0:
                 g = self.parse_rdf(extructed['rdfa'], 'json-ld', log_msg='HTML embedded RDFa')
                 if len(g) > 0:
+                    self.info(f"Found RDFa metadata embedded in the HTML with extruct")
                     return g
                 elif not metadata_obj:
                     metadata_obj = extructed['rdfa']
@@ -196,7 +203,7 @@ class FairTestEvaluation(BaseModel):
             # The rest is not extracted because usually give no interesting metadata:
             # opengraph, microformat
         except Exception as e:
-            self.info('Error when running extruct on ' + url + '. Getting: ' + str(e.args[0]))
+            self.info(f"Error when running extruct on {url}. Getting: {str(e.args[0])}")
 
 
         # Perform content negociation last because it's the slowest for a lot of URLs like zenodo
@@ -208,24 +215,20 @@ class FairTestEvaluation(BaseModel):
                 r = requests.get(url, headers={'accept': mime_type})
                 r.raise_for_status()  # Raises a HTTPError if the status is 4xx, 5xxx
                 content_type = r.headers['Content-Type'].replace(' ', '').replace(';charset=utf-8', '')
-                # If return text/plain we parse as turtle
-                content_type = content_type.replace('text/plain', 'text/turtle')
-                self.info(f'Found some metadata in {content_type} when asking for {mime_type}')
-                if content_type.startswith('text/html'):
-                    # If HTML we check later with extruct
-                    html_text = r.text
-                    continue
+                # If return text/plain we parse as turtle or JSON-LD
+                # content_type = content_type.replace('text/plain', 'text/turtle')
+                self.info(f'Content-negotiation: found some metadata in {content_type} when asking for {mime_type}')
                 try:
-                    # If return JSON-LD
+                    # If returns JSON
                     self.data['json-ld'] = r.json()
                     if not metadata_obj:
                         metadata_obj = r.json()
-                    return self.parse_rdf(r.json(), content_type, log_msg='content negotiation RDF')
+                    return self.parse_rdf(r.json(), 'json-ld', log_msg='content negotiation JSON-LD RDF')
                 except Exception:
-                    # If returns RDF, such as turtle
+                    # If returns RDF as text, such as turtle
                     return self.parse_rdf(r.text, content_type, log_msg='content negotiation RDF')
-            except Exception:
-                self.info(f'Could not find metadata with content-negotiation when asking for: {mime_type}')
+            except Exception as e:
+                self.info(f'Content-negotiation: error with {url} when asking for {mime_type}. Getting {str(e.args[0])}')
                 # Error: e.args[0]
 
         return metadata_obj
@@ -248,15 +251,14 @@ class FairTestEvaluation(BaseModel):
             g (Graph): A RDFLib Graph
         """
         # https://rdflib.readthedocs.io/en/stable/plugin_parsers.html
-        # rdflib_formats = ['turtle', 'json-ld', 'xml', 'ntriples', 'nquads', 'trig', 'n3']
-        # We need to make this ugly fix because regular content negotiation dont work with schema.org
-        # https://github.com/schemaorg/schemaorg/issues/2578
+        parse_formats = ['turtle', 'json-ld', 'xml', 'ntriples', 'nquads', 'trig', 'n3']
+
         if type(rdf_data) == dict:
             rdf_data = [rdf_data]
         if type(rdf_data) == list:
             for rdf_entry in rdf_data:
                 try:
-                    # Dirty hack to fix RDFLib that is not able to parse JSON-LD schema.org:
+                    # Dirty hack to fix RDFLib that is not able to parse JSON-LD schema.org (https://github.com/schemaorg/schemaorg/issues/2578)
                     if '@context' in rdf_entry:
                         if isinstance(rdf_entry['@context'], str):
                             if rdf_entry['@context'].startswith('http://schema.org') or rdf_entry['@context'].startswith('https://schema.org'):
@@ -270,16 +272,42 @@ class FairTestEvaluation(BaseModel):
             # RDFLib JSON-LD had issue with encoding: https://github.com/RDFLib/rdflib/issues/1416
             rdf_data = jsonld.expand(rdf_data)
             rdf_data = json.dumps(rdf_data)
-            mime_type = 'json-ld'
+            parse_formats = ['json-ld']
         
-        g = ConjunctiveGraph()
-        try:
-            g.parse(data=rdf_data, format=mime_type)
-            self.info(f'{str(len(g))} triples parsed. Metadata from {mime_type} {log_msg} parsed with RDFLib parser {mime_type}')
-        except Exception as e:
-            self.warn('Could not parse ' + mime_type + ' metadata from ' + log_msg + ' with RDFLib parser ' + mime_type + ' ' + str(e))
+        else:
+            # Try to guess the format to parse from mime type
+            mime_type = mime_type.split(';')[0]
+            if 'turtle' in mime_type:
+                parse_formats = ['turtle']
+            elif 'xml' in mime_type:
+                parse_formats = ['xml']
+            elif 'ntriples' in mime_type:
+                parse_formats = ['ntriples']
+            elif 'nquads' in mime_type:
+                parse_formats = ['nquads']
+            elif 'trig' in mime_type:
+                parse_formats = ['trig']
+            # elif mime_type.startswith('text/html'):
+            #     parse_formats = []
 
+        g = ConjunctiveGraph()
+        # Remove some auto-generated triples about the HTML content
+        remove_preds = ['http://www.w3.org/1999/xhtml/vocab#role']
+        for rdf_format in parse_formats:
+            try:
+                g = ConjunctiveGraph()
+                g.parse(data=rdf_data, format=rdf_format)
+
+                for rm_pred in remove_preds:
+                    g.remove((None, URIRef(rm_pred), None))
+
+                self.info(f'Successfully parsed {mime_type} RDF from {log_msg} with parser {rdf_format}, containing {str(len(g))} triples')
+                return g
+            except Exception as e:
+                self.info(f'Could not parse {mime_type} metadata from {log_msg} with parser {rdf_format}. Getting error: {str(e)}')
+        
         return g
+        # return None
 
 
     def extract_prop(self, 
@@ -308,17 +336,16 @@ class FairTestEvaluation(BaseModel):
             elif str(pred).startswith('https://'):
                 check_preds.add(URIRef(str(pred).replace('https://', 'http://')))
 
-        self.info(f"Checking properties values for properties: {preds}")
-        if subj:
-            self.info(f"Checking properties values for subject URI(s): {str(subj)}")
-
+        # self.info(f"Checking properties values for properties: {preds}")
+        # if subj:
+        #     self.info(f"Checking properties values for subject URI(s): {str(subj)}")
         for pred in list(check_preds):
             if not isinstance(subj, list):
                 subj = [subj]
                 # test_subjs = [URIRef(str(s)) for s in subj] 
             for test_subj in subj:
                 for s, p, o in g.triples((test_subj, URIRef(str(pred)), None)):
-                    self.info(f"Found a value for property {str(pred)} => {str(o)}")
+                    self.info(f"Found a value for a property {str(pred)} => {str(o)}")
                     values.add(o)
 
         return list(values)
@@ -358,7 +385,7 @@ class FairTestEvaluation(BaseModel):
             uri_ref = URIRef(str(alt_uri))
             # Search with the subject URI as triple subject
             for s, p, o in g.triples((uri_ref, None, None)):
-                self.info(f"Found subject identifier in metadata: {str(s)}")
+                self.info(f"Found the subject URI in the metadata: {str(s)}")
                 resource_properties[str(p)] = str(o)
                 subject_uri = uri_ref
 
@@ -366,14 +393,14 @@ class FairTestEvaluation(BaseModel):
                 # Search with the subject URI as triple object
                 for pred in all_preds_uris:
                     for s, p, o in g.triples((None, pred, uri_ref)):
-                        self.info(f"Found subject identifier in metadata: {str(s)}")
+                        self.info(f"Found the subject URI in the metadata: {str(s)}")
                         resource_linked_to[str(s)] = str(p)
                         subject_uri = s
 
                     if not subject_uri:
-                        # Also check when URI defined as Literal
+                        # Also check when the subject URI defined as Literal
                         for s, p, o in g.triples((None, pred, Literal(str(uri_ref)))):
-                            self.info(f"Found subject identifier in metadata: {str(s)}")
+                            self.info(f"Found the subject URI in the metadata: {str(s)}")
                             resource_linked_to[str(s)] = str(p)
                             subject_uri = s
 
@@ -428,6 +455,7 @@ class FairTestEvaluation(BaseModel):
             "https://schema.org/contentUrl", 
             "http://www.w3.org/ns/dcat#downloadURL"
         ]
+        self.info(f"Checking if the data URI point to a download URL using one of the following predicates: {', '.join(content_props)}")
         extracted_urls = set()
         for data_uri in data_uris:
             if isinstance(data_uri, BNode):
